(ns com.aurellem.gb.characters
  (:use (com.aurellem.gb gb-driver constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))


(def pronouns
  ;; Note: 0x0 seems to be the end-of-name sentinel
  {
   ;; what is 0x57? what is 0x0?
   :RED [0x52]
   :BLUE [0x53]
   :opponent [0x01 0x49 0xD0 0x0]
   :turn-pokemon [0x5A]
   :off-turn-pokemon [0x59]
   :your-pokemon [0x01 0x08 0xD0 0x0]
   :opponent-pokemon [0x50 0x01 0xD9 0xCF 0x0]
   :n-times [0x50 0x09 0x05 0xCD 0x11 0x0]
   })



(def meta-characters
  {0x00 "non-meta text follows"
   0x01 "load text from RAM (nn)"
   0x02 "BCD number (nn)C" ;; C is 3 flags and 5 length bits."
   0x03 "store text in RAM (nn)"
   0x04 "draw box (nn)WH"
   0x05 "BEGIN text on textbox line 2"
   0x06 "wait for A or B"
   0x07 "scrolling text"
   0x08 "BEGIN routine"
   0x09 "load RAM (nn) hex-to-dec. (nn)BC. B is bytes to read, C is
  digits to display"
   0x0A "wait 30 vblanks iff you pressed A or B"

   0x0B "sound: got item"
   0x0E "sound: learned skill"
   0x0F "sound: got rarecandy" ;; incidentally, same as 'got item'
   0x10 "sound: caught pokemon!"
   0x11 "sound: gave OAK the parcel"
   0x12 "sound: same as 0x10???"
   0x13 "sound: new data added"
   0x14 "sound: cry-nidorina"
   0x15 "sound: cry-pidgeot"
   0x16 "sound: cry-dewgong"

   0x4E "new line"
   0x51 "new page"
   0x55 "scroll page"
   0x50 "END command" ;; the pair 0x50 0x50 ??
   0x57 "END command"
   0x58 "END after you press A or B"
   })




(def end-of-name-sentinel 0x00)

(def character-code->character
  {
   0x0 "" ;; separator character?
   0x58 "\n"
   ;;0x00   "<";;"end-of-name-sentinel" ;; begin messsage
   ;;0x49   "\n //" ;; ocsenave: pagebreak pokedex
   ;;0x4E   "\n..." ; ocsenave: clearscroll pokedex page
   ;;0x4F   "\n" ; newline
   ;;0x50   "#";;"end-of-pokemon-name-sentinel"
   ;;0x51   "\n\n" ;; ocsenave: clear screen
   0x52 "[RED]" ;;ocsenave: placeholder for your name?
   0x54   "[POKE]"
   ;;0x55   "_" ;; ocsenave: breaking space?
   ;0x57   ">" ;; ocsenave: end message
   ;0x5F   ">" ;; ocsenave: end pokedex entry??
   0x60   "A-bold"
   0x61   "B-bold"
   0x62   "C-bold"
   0x63   "D-bold"
   0x64   "E-bold"
   0x65   "F-bold"
   0x66   "G-bold"
   0x67   "H-bold"
   0x68   "I-bold"
   0x69   "V-bold"
   0x6A   "S-bold"
   0x6B   "L-bold"
   0x6C   "M-bold"
   0x7F   " " ;space
   0x80   "A"
   0x81   "B"
   0x82   "C"
   0x83   "D"
   0x84   "E"
   0x85   "F"
   0x86   "G"
   0x87   "H"
   0x88   "I"
   0x89   "J"
   0x8A   "K"
   0x8B   "L"
   0x8C   "M"
   0x8D   "N"
   0x8E   "O"
   0x8F   "P"
   0x90   "Q"
   0x91   "R"
   0x92   "S"
   0x93   "T"
   0x94   "U"
   0x95   "V"
   0x96   "W"
   0x97   "X"
   0x98   "Y"
   0x99   "Z"
   0x9A   "("
   0x9B   ")"
   0x9C   ":"
   0x9D   ";"
   0xA0   "a"
   0xA1   "b"
   0xA2   "c"
   0xA3   "d"
   0xA4   "e"
   0xA5   "f"
   0xA6   "g"
   0xA7   "h"
   0xA8   "i"
   0xA9   "j"
   0xAA   "k"
   0xAB   "l"
   0xAC   "m"
   0xAD   "n"
   0xAE   "o"
   0xAF   "p"
   0xB0   "q"
   0xB1   "r"
   0xB2   "s"
   0xB3   "t"
   0xB4   "u"
   0xB5   "v"
   0xB6   "w"
   0xB7   "x"
   0xB8   "y"
   0xB9   "z"
   0xBA   "é" 
   0xBB   "'d" ;ocsenave
   0xBC   "'l" ;ocsenave
   0xBD   "'s" ;ocsenave
   0xBE   "'t" ;ocsenave
   0xBF   "'v" ; ocsenave
   
   0xE0   "'"
   0xE1   "PK"
   0xE2   "MN"
   0xE3   "-"
   0xE4   "'r" ;ocsenave
   0xE5   "'m" ;ocsenave
   0xE6   "?"
   0xE7   "!"
   0xE8   "."
   0xEF   "♂"
   0xF0   "$" ;;"pokemon-money-symbol"
   0xF1   "."
   0xF2   "/"
   0xF3   ","
   0xF4   ", "
   0xF5   "♀" 
   0xF6   "0 "
   0xF7   "1"
   0xF8   "2"
   0xF9   "3"
   0xFA   "4"
   0xFB   "5"
   0xFC   "6"
   0xFD   "7"
   0xFE   "8"
   0xFF   "9"
   })

(def character->character-code
  (zipmap (vals character-code->character)
          (keys character-code->character)))

(defn str->character-codes [s]
  (map character->character-code (map str s)))

(defn character-codes->str [codes]
  (apply str
         (map #(character-code->character
                %
                (format "[0x%02X]" %))
              codes)))

(defn print-text
  ([^SaveState state begin end]
     (dorun
      (map (fn [character-code line]
             (println
              (format "0x%04X: " line)
              (str (character-code->character character-code))))
           (subvec (vec (memory state)) begin end)
           (range begin end)))
     state)
  ([begin end]
     (print-text @current-state begin end)))

(defn read-name [codes]
  (character-codes->str
   (take-while
    (partial not= end-of-name-marker) codes)))
